#!/bin/bash
#
# RoksNet Portal application Apache Tomcat and Apache2 configuration
#
# Aktors 2016

xrd_prefix=/usr/xtee
tomcat_home=/var/lib/tomcat7 
tomcat_share_home=/usr/share/tomcat7
apache2_home=/etc/apache2
# 'y' to skip estonian portal related prompt questions, 'n' to include them; value could be replaced before package generation
skip_estonian=y
echo "Before proceed please make sure you have installed Apache2 with mod_jk and Apache Tomcat (apt-get install apache2 libapache2-mod-jk)"

if [ ! -d $tomcat_home/webapps ]
then
	echo -n "Please provide Apache Tomcat server working directory [$tomcat_home]? "
	read user_tomcat < /dev/tty
	if [ "$user_tomcat" == "" ]
	then
		user_tomcat=$tomcat_home
	fi
	tomcat_home=$user_tomcat
	
fi
if [ ! -d $tomcat_home ]
then
	echo "$tomcat_home is not found"
	exit 1
fi

if [ ! -d $apache2_home ]
	then
	echo -n "Please provide Apache2 working directory [$apache2_home]? "
	read user_apache2 < /dev/tty
	if [ "$user_apache2" == "" ]
	then
		user_apache2=$apache2_home
	fi
	apache2_home=$user_apache2
	
	if [ ! -d $user_apache2 ]
	then
		echo "$apache2_home is not found"
		exit 1
	fi
fi
if [ ! -d $apache2_home ]
then
	echo "$apache2_home is not found"
	exit 1
fi

# Check if Apache 2 server is running. If it's not, attempt to start.
status_adverb=
while ! /etc/init.d/apache2 status > /dev/null # do not show output, too verbose
do
	echo "Apache2 service is not running, attempting to start it."
	/etc/init.d/apache2 start
	status_adverb=" now"
	sleep 1
done
echo "Apache2 service is$status_adverb running."

if [ -f /etc/default/tomcat7 ]
then
	echo "Adding memory to Tomcat..."
	grep -q -e 'MaxPermSize' /etc/default/tomcat7 || echo 'JAVA_OPTS="${JAVA_OPTS} -Xms1g -Xmx1g -XX:MaxPermSize=256m"' >> /etc/default/tomcat7
	grep -q -e 'Xms1g' /etc/default/tomcat7 || echo 'JAVA_OPTS="${JAVA_OPTS} -Xms1g"' >> /etc/default/tomcat7
	grep -q -e 'Xms1g' /etc/default/tomcat7 || echo 'JAVA_OPTS="${JAVA_OPTS} -Xmx1g"' >> /etc/default/tomcat7
	
	# replace JAVA_HOME variable in tomcat7 configuration with environment variable if that exists
	if [ -d "$JAVA_HOME" ] && grep -q '#JAVA_HOME' /etc/default/tomcat7
	then
		echo "Replacing JAVA_HOME in /etc/default/tomcat7 with $JAVA_HOME and commenting it in"
		# regex replace
		#  - separator is ':'
		#  - only replace when JAVA_HOME is commented out like initially (#JAVA_HOME)
		#  - take replacement value from JAVA_HOME env variable and remove comment-out prefix #
        #  - g: replace all instances
		sed -ie 's:#JAVA_HOME=.*:JAVA_HOME="'$JAVA_HOME'":g' /etc/default/tomcat7
	fi
fi

echo "Replacing server.xml..."
#replace server.xml
cp $xrd_prefix/apache2/server.xml $tomcat_home/conf/
#remove tomcat ROOT app
rm -rf $tomcat_home/webapps/ROOT

echo "Replacing mod jk.conf..."
### add mod-jk.conf
cp $xrd_prefix/apache2/jk.conf $apache2_home/mods-available/

echo "Enabling Apache2 mods..."
### enable mods (if not enabled yet)
a2enmod jk rewrite ssl headers proxy_http

if [ -f $apache2_home/sites-available/ssl.conf ]
then
	echo -n "Do you want to overwrite Apache2 configuration with default settings? [y/n] [default: n]: "
	read user_overwrite < /dev/tty
	if [ `echo $user_overwrite | grep -i y ` ]
	then
		echo "Overwriting Apache2 server conf at $apache2_home/sites-available/ssl.conf..."
	    sed -n '/admin/, /Location/p' $apache2_home/sites-available/ssl.conf | grep Allow > /tmp/ssl.allowed.tmp
		sed -i '/admin/, /Location/{
		 /Allow./{
		 s/.//g
		 r /tmp/ssl.allowed.tmp
		 }
		}' $xrd_prefix/apache2/ssl.conf
		cp $xrd_prefix/apache2/ssl.conf $apache2_home/sites-available/ssl.conf
	fi
	
else
	echo "Copying Apache2 server conf to $apache2_home/sites-available/ssl.conf..."
	cp $xrd_prefix/apache2/ssl.conf $apache2_home/sites-available/ssl.conf
	
	a2ensite ssl.conf
	a2dissite 000-default
fi


#certs
echo "Updating certificate scripts... "
if [ ! -d $apache2_home/ssl ]
then 
	mkdir $apache2_home/ssl
fi

cd $apache2_home/ssl

cp $xrd_prefix/apache2/updatecrl.sh $apache2_home/ssl
cp $xrd_prefix/apache2/create_ca_cert.sh $apache2_home/ssl
cp $xrd_prefix/apache2/create_server_cert.sh $apache2_home/ssl
cp $xrd_prefix/apache2/misp2.cnf $apache2_home/ssl
cp $xrd_prefix/apache2/create_sslproxy_cert.sh $apache2_home/ssl

chmod 755 $xrd_prefix/apache2/cleanXFormsDir.sh
chmod 755 create_*_cert.sh

if [[ ! -f $apache2_home/ssl/ca.crl || ! -f $apache2_home/ssl/MISP2_CA_key.der ]]
then
	echo "Creating CA certificate... "
	./create_ca_cert.sh
else
	echo "CA certificate already created... "
fi

if [[ ! -f $apache2_home/ssl/httpsd.cert || ! -f $apache2_home/ssl/httpsd.key ]]
then
	echo "Creating server certificate... "
	./create_server_cert.sh
else
	echo "Server certificate already created... "
fi

key_access_rights="$(ls -l $apache2_home/ssl/httpsd.key | cut -c 1-10)"
if [[ "$key_access_rights" == "-rw-r--r--" ]] # compare to default access rights
then
	echo "Changing server private key access rights to -r-------- (previously $key_access_rights)."
	chmod 400 $apache2_home/ssl/httpsd.key
fi

# Only prompt when estonian portal related questions are not skipped
if [ "$skip_estonian" != "y" ]
then
echo -n "Do you want to update SK certificates (y/n)? [y] "
read sk_certs < /dev/tty
if [ "$sk_certs" == "" ]
then
        sk_certs="y"
fi

if [ `echo $sk_certs | grep -i y ` ]
then

	echo "Downloading root certificates... "
	wget -O sk_root_2011_crt.pem http://www.sk.ee/upload/files/EECCRCA.pem.cer
	wget -O sk_esteid_2011_crt.pem http://www.sk.ee/upload/files/ESTEID-SK%202011.pem.cer
	wget -O sk_esteid_2015_crt.pem http://www.sk.ee/upload/files/ESTEID-SK_2015.pem.crt

	# OCSP refresh
	echo "Downloading OCSP certs... "
	wget -O sk_esteid_ocsp_2011.pem http://www.sk.ee/upload/files/SK%20OCSP%20RESPONDER%202011.pem.cer

	cat sk_esteid_ocsp_2011.pem >> sk_esteid_ocsp.pem
	rm -f sk_esteid_ocsp_2011.pem

	# update crl 
	 echo "Updating CRL..."
	./updatecrl.sh

fi
fi
c_rehash ./

/etc/init.d/apache2 restart

echo "Successfully installed roksnet-base package"
